@@ -749,18 +749,22 @@ def is_direct_child(_: Optional[DerivationTree], path_1: Path, path_2: Path) ->
 
 DIRECT_CHILD_PREDICATE = StructuralPredicate("direct_child", 2, is_direct_child)
 
-STANDARD_STRUCTURAL_PREDICATES = {
-    AFTER_PREDICATE,
-    BEFORE_PREDICATE,
-    CONSECUTIVE_PREDICATE,
-    DIFFERENT_POSITION_PREDICATE,
-    DIRECT_CHILD_PREDICATE,
-    IN_TREE_PREDICATE,
-    LEVEL_PREDICATE,
-    SAME_POSITION_PREDICATE,
-    NTH_PREDICATE,
-}
+STANDARD_STRUCTURAL_PREDICATES = frozenset(
+    {
+        AFTER_PREDICATE,
+        BEFORE_PREDICATE,
+        CONSECUTIVE_PREDICATE,
+        DIFFERENT_POSITION_PREDICATE,
+        DIRECT_CHILD_PREDICATE,
+        IN_TREE_PREDICATE,
+        LEVEL_PREDICATE,
+        SAME_POSITION_PREDICATE,
+        NTH_PREDICATE,
+    }
+)
 
-STANDARD_SEMANTIC_PREDICATES = {
-    COUNT_PREDICATE,
-}
+STANDARD_SEMANTIC_PREDICATES = frozenset(
+    {
+        COUNT_PREDICATE,
+    }
+)
